import { FetchCache, LocalCacheState } from './cache';
import { FetchAction } from './action';
import mitt from 'mitt';
export interface FetchDispatcher<TResult extends FetchDispatcherResult<any, any>> {
    (action: FetchAction<any>): Promise<TResult>;
}
export declare type FetchDispatcherResult<TData, TContext = {}> = {
    error?: Error;
    data?: TData;
} & TContext;
export interface HeadersObject {
    [headerName: string]: any;
}
export declare type DefaultFetchDispatcherResult<TData> = FetchDispatcherResult<TData, {
    status?: number;
    headers?: HeadersObject;
}>;
export declare const defaultDispatcher: FetchDispatcher<DefaultFetchDispatcherResult<any>>;
export interface IdFromActionFn {
    (action: FetchAction<any>): string;
}
export interface ShouldCacheFetchResultFn {
    (action: FetchAction<any>): boolean;
}
interface EventHandlersMap {
    update?: (newValue: any) => void;
    revalidate?: () => void;
}
export declare class FetchClient<TDispatcherResult extends FetchDispatcherResult<any, any>> {
    cache: FetchCache<TDispatcherResult, any>;
    protected dispatcher: FetchDispatcher<TDispatcherResult>;
    /**
     * A function that creates an id from an action
     *
     * @type {IdFromActionFn}
     * @memberof FetchClient
     */
    idFromAction: IdFromActionFn;
    /**
     * A function that tells the client if it should cache the action result
     *
     * @type {ShouldCacheFetchResultFn}
     * @memberof FetchClient
     */
    shouldCacheFetchResult: ShouldCacheFetchResultFn;
    /**
     * An object that matches the action id to its promise on queue
     *
     * @protected
     * @type {{ [fetchId in string]: Promise<any> }}
     * @memberof FetchClient
     */
    protected queueMap: {
        [fetchId in string]: Promise<TDispatcherResult>;
    };
    /**
     * Mitt instance that handles all events
     * @protected
     * @type {mitt.Emitter}
     * @memberof FetchClient
     */
    protected emitter: mitt.Emitter;
    constructor(cache: FetchCache<TDispatcherResult, any>, dispatcher: FetchDispatcher<TDispatcherResult>, idFromAction?: IdFromActionFn, shouldCacheAction?: ShouldCacheFetchResultFn);
    /**
     * Dispatchs an action and adds it to queue. If an equal action, compared by id, is found
     * we reuse the same promise and return it. When resolved we emit to all listeners that
     * the value has changed.
     * @param action Action to be dispatch
     */
    dispatch(action: FetchAction<any>, givenCacheId?: string): Promise<TDispatcherResult>;
    /**
     *
     * @param fetchId
     * @param handlersMap Map of event handle
     * @returns A function that removes the listener
     */
    listen(fetchId: string, handlersMap: EventHandlersMap): () => void;
    /**
     * Get an item from cache by it's fetch id
     * @param fetchId
     */
    get(fetchId: string): TDispatcherResult | undefined;
    /**
     * Create or update a value on cache
     * @param fetchId
     * @param newResult New value to be set to `data` property
     */
    set(fetchId: string, newValueOrUpdateResultFn: ((prevValue?: TDispatcherResult) => TDispatcherResult) | TDispatcherResult): void;
    get queue(): Promise<TDispatcherResult>[];
    /**
     * Waits until the running fetchs are resolved
     */
    waitForFetchs(): Promise<void>;
}
interface CreateClientConfig {
    cache?: FetchCache<FetchDispatcherResult<any, any>, any>;
    initialCacheState?: LocalCacheState;
    dispatcher?: FetchDispatcher<any>;
}
export declare const createClient: ({ cache, initialCacheState, dispatcher, }: CreateClientConfig) => FetchClient<any>;
export {};
